<?php

namespace Litecms\Advert\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Advert\Forms\Advert as AdvertForm;
use Litecms\Advert\Http\Requests\AdvertResourceRequest;
use Litecms\Advert\Http\Resources\AdvertResource;
use Litecms\Advert\Http\Resources\AdvertsCollection;
use Litecms\Advert\Models\Advert;
use Litecms\Advert\Actions\AdvertAction;
use Litecms\Advert\Actions\AdvertActions;


/**
 * Resource controller class for advert.
 */
class AdvertResourceController extends BaseController
{

    /**
     * Initialize advert resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = AdvertForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.advert.modules'), 'advert', guard_url('advert'));
            return $next($request);
        });
    }

    /**
     * Display a list of advert.
     *
     * @return Response
     */
    public function index(AdvertResourceRequest $request)
    {
        $request = $request->all();
        $page = AdvertActions::run('paginate', $request);

        $data = new AdvertsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('advert::advert.names'))
            ->view('advert::advert.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display advert.
     *
     * @param Request $request
     * @param Model   $advert
     *
     * @return Response
     */
    public function show(AdvertResourceRequest $request, Advert $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AdvertResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('advert::advert.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('advert::advert.show')
            ->output();
    }

    /**
     * Show the form for creating a new advert.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AdvertResourceRequest $request, Advert $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AdvertResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('advert::advert.name'))
            ->view('advert::advert.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new advert.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AdvertResourceRequest $request, Advert $model)
    {
        try {
            $request = $request->all();
            $model = AdvertAction::run('store', $model, $request);
            $data = new AdvertResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('advert::advert.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('advert/advert/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/advert/advert'))
                ->redirect();
        }

    }

    /**
     * Show advert for editing.
     *
     * @param Request $request
     * @param Model   $advert
     *
     * @return Response
     */
    public function edit(AdvertResourceRequest $request, Advert $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AdvertResource($model);
        // return view('advert::advert.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('advert::advert.name'))
            ->view('advert::advert.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the advert.
     *
     * @param Request $request
     * @param Model   $advert
     *
     * @return Response
     */
    public function update(AdvertResourceRequest $request, Advert $model)
    {
        try {
            $request = $request->all();
            $model = AdvertAction::run('update', $model, $request);
            $data = new AdvertResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('advert::advert.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('advert/advert/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advert/advert/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the advert.
     *
     * @param Model   $advert
     *
     * @return Response
     */
    public function destroy(AdvertResourceRequest $request, Advert $model)
    {
        try {

            $request = $request->all();
            $model = AdvertAction::run('destroy', $model, $request);
            $data = new AdvertResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('advert::advert.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('advert/advert/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advert/advert/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
