<?php

namespace Litecms\Advert\Http\Controllers;

use Exception;
use Litecms\Advert\Actions\AdvertWorkflow;
use Litecms\Advert\Http\Requests\AdvertWorkflowRequest;
use Litecms\Advert\Http\Resources\AdvertResource;
use Litecms\Advert\Models\Advert;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class AdvertWorkflowController extends BaseController
{
    /**
     * Action controller function for advert.
     *
     * @param Model $advert
     * @param action next action for the advert.
     *
     * @return Response
     */
    public function __invoke(AdvertWorkflowRequest $request, Advert $advert, $trasition)
    {
        try {
            $request = $request->all();
            $advert = AdvertWorkflow::run($trasition, $advert, $request);
            $data = new AdvertResource($advert);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('advert::advert.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('advert/advert/' . $advert->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advert/advert/' . $advert->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the advert and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $advert
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $advert, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $advert = Advert::findBySignedId($advert);
        $transitions = $this->transitions($advert, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('advert::advert.name') . ' ' . trans('Approvals'))
            ->view('advert::public.advert.workflow')
            ->layout('mobile')
            ->data(compact('advert', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the advert.
     *
     * @param PublicRequest $request
     * @param string $advert
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $advert, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $advert = Advert::findBySignedId($advert);
        $user = user();
        $advert = AdvertWorkflow::run($transition, $advert, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($advert, $user)
    {
        $transitions = [];
        foreach ($advert->workflow()->transitions($advert) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.advert.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'advert' => $advert->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $advert->workflow()->form($value);
            $array['label'] = trans('advert::advert.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
